'use strict';

var React = require('react');

function _arrayLikeToArray(r, a) {
  (null == a || a > r.length) && (a = r.length);
  for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e];
  return n;
}
function _arrayWithHoles(r) {
  if (Array.isArray(r)) return r;
}
function _iterableToArrayLimit(r, l) {
  var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"];
  if (null != t) {
    var e,
      n,
      i,
      u,
      a = [],
      f = true,
      o = false;
    try {
      if (i = (t = t.call(r)).next, 0 === l) ; else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0);
    } catch (r) {
      o = true, n = r;
    } finally {
      try {
        if (!f && null != t.return && (u = t.return(), Object(u) !== u)) return;
      } finally {
        if (o) throw n;
      }
    }
    return a;
  }
}
function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _slicedToArray(r, e) {
  return _arrayWithHoles(r) || _iterableToArrayLimit(r, e) || _unsupportedIterableToArray(r, e) || _nonIterableRest();
}
function _unsupportedIterableToArray(r, a) {
  if (r) {
    if ("string" == typeof r) return _arrayLikeToArray(r, a);
    var t = {}.toString.call(r).slice(8, -1);
    return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : undefined;
  }
}

var languages = {
	javascript: {
		keywords: {
			controlFlow: [
				"break",
				"case",
				"catch",
				"continue",
				"default",
				"do",
				"else",
				"finally",
				"for",
				"if",
				"return",
				"switch",
				"throw",
				"try",
				"while",
				"with"
			],
			declarations: [
				"class",
				"const",
				"function",
				"let",
				"var"
			],
			operators: [
				"delete",
				"instanceof",
				"new",
				"typeof",
				"void",
				"yield"
			],
			special: [
				"debugger",
				"export",
				"extends",
				"import",
				"super",
				"this"
			]
		},
		comments: {
			singleLine: "//",
			multiLine: [
				"/*",
				"*/"
			]
		},
		strings: {
			delimiters: [
				"\"",
				"'",
				"`"
			]
		},
		numbers: "\\b\\d+(\\.\\d+)?\\b",
		operators: [
			"+",
			"-",
			"*",
			"/",
			"%",
			"++",
			"--",
			"==",
			"!=",
			"===",
			"!==",
			">",
			"<",
			">=",
			"<=",
			"&&",
			"||",
			"!",
			"=",
			"+=",
			"-=",
			"*=",
			"/=",
			"%=",
			"=>"
		]
	},
	python: {
		keywords: {
			controlFlow: [
				"else",
				"elif",
				"if",
				"try",
				"except",
				"finally",
				"for",
				"while",
				"continue",
				"break"
			],
			declarations: [
				"def",
				"class",
				"lambda",
				"global",
				"nonlocal"
			],
			boolean: [
				"True",
				"False",
				"None"
			],
			operators: [
				"and",
				"or",
				"not",
				"is",
				"in"
			]
		},
		comments: {
			singleLine: "#"
		},
		strings: {
			delimiters: [
				"\"",
				"'",
				"'''",
				"\"\"\""
			]
		},
		numbers: "\\b\\d+(\\.\\d+)?\\b",
		operators: [
			"+",
			"-",
			"*",
			"/",
			"%",
			"//",
			"**",
			"==",
			"!=",
			">",
			"<",
			">=",
			"<=",
			"and",
			"or",
			"not",
			"is",
			"in"
		]
	},
	java: {
		keywords: {
			controlFlow: [
				"if",
				"else",
				"for",
				"while",
				"switch",
				"case",
				"default",
				"break",
				"continue",
				"return",
				"throw",
				"try",
				"catch",
				"finally"
			],
			declarations: [
				"class",
				"interface",
				"enum",
				"extends",
				"implements",
				"new",
				"import",
				"package"
			],
			modifiers: [
				"public",
				"protected",
				"private",
				"static",
				"final",
				"abstract",
				"synchronized",
				"volatile",
				"transient"
			],
			primitives: [
				"int",
				"long",
				"short",
				"byte",
				"float",
				"double",
				"char",
				"boolean",
				"void"
			]
		},
		comments: {
			singleLine: "//",
			multiLine: [
				"/*",
				"*/"
			]
		},
		strings: {
			delimiters: [
				"\"",
				"'"
			]
		},
		numbers: "\\b\\d+(\\.\\d+)?(f|d|F|D)?\\b",
		operators: [
			"+",
			"-",
			"*",
			"/",
			"%",
			"++",
			"--",
			"==",
			"!=",
			">",
			"<",
			">=",
			"<=",
			"&&",
			"||",
			"!",
			"=",
			"+=",
			"-=",
			"*=",
			"/=",
			"%=",
			"&",
			"|",
			"^",
			"~",
			"<<",
			">>",
			">>>"
		]
	},
	c: {
		keywords: {
			controlFlow: [
				"if",
				"else",
				"switch",
				"case",
				"default",
				"break",
				"continue",
				"return",
				"goto",
				"while",
				"do",
				"for"
			],
			declarations: [
				"auto",
				"extern",
				"register",
				"static",
				"typedef",
				"struct",
				"union",
				"enum"
			],
			types: [
				"void",
				"char",
				"short",
				"int",
				"long",
				"float",
				"double",
				"signed",
				"unsigned",
				"const",
				"volatile"
			],
			special: [
				"sizeof",
				"_Alignas",
				"_Alignof",
				"_Atomic",
				"_Bool",
				"_Complex",
				"_Generic",
				"_Imaginary",
				"_Noreturn",
				"_Static_assert",
				"_Thread_local"
			]
		},
		comments: {
			singleLine: "//",
			multiLine: [
				"/*",
				"*/"
			]
		},
		strings: {
			delimiters: [
				"\"",
				"'"
			]
		},
		numbers: "\\b\\d+(\\.\\d+)?(f|F|l|L)?\\b",
		operators: [
			"+",
			"-",
			"*",
			"/",
			"%",
			"++",
			"--",
			"==",
			"!=",
			">",
			"<",
			">=",
			"<=",
			"&&",
			"||",
			"!",
			"=",
			"+=",
			"-=",
			"*=",
			"/=",
			"%=",
			"&",
			"|",
			"^",
			"~",
			"<<",
			">>",
			"->"
		]
	},
	ruby: {
		keywords: {
			controlFlow: [
				"if",
				"elsif",
				"else",
				"unless",
				"while",
				"until",
				"for",
				"break",
				"next",
				"redo",
				"retry",
				"return",
				"yield",
				"rescue",
				"ensure"
			],
			declarations: [
				"def",
				"class",
				"module",
				"alias",
				"begin",
				"end"
			],
			constants: [
				"true",
				"false",
				"nil",
				"self",
				"super"
			]
		},
		comments: {
			singleLine: "#"
		},
		strings: {
			delimiters: [
				"\"",
				"'",
				"%q",
				"%Q"
			]
		},
		numbers: "\\b\\d+(\\.\\d+)?\\b",
		operators: [
			"+",
			"-",
			"*",
			"/",
			"%",
			"**",
			"==",
			"!=",
			"<",
			">",
			"<=",
			">=",
			"&&",
			"||",
			"!",
			"=",
			"+=",
			"-=",
			"*=",
			"/=",
			"%=",
			"..",
			"..."
		]
	}
};
var syntaxData = {
	languages: languages
};

document.head.appendChild(document.createElement("style")).textContent=":root .standard{--bg-color:#2e3440;--text-color:#d8dee9;--keyword-controlFlow:#81a1c1;--keyword-declarations:#b48ead;--operator-color:#d08770;--number-color:#a3be8c;--string-color:#ebcb8b;--comment-color:#616e88;--identifier-color:#d8dee9;}:root .solarized-light{--bg-color:#fdf6e3;--text-color:#002b36;--keyword-controlFlow:#b58900;--keyword-declarations:#cb4b16;--operator-color:#dc322f;--number-color:#2aa198;--string-color:#268bd2;--comment-color:#859900;--identifier-color:#002b36;}:root .gruvbox-dark{--bg-color:#282828;--text-color:#ebdbb2;--keyword-controlFlow:#fb4934;--keyword-declarations:#b8bb26;--operator-color:#fe8019;--number-color:#fabd2f;--string-color:#83a598;--comment-color:#928374;--identifier-color:#ebdbb2;}:root .monokai{--bg-color:#272822;--text-color:#f8f8f2;--keyword-controlFlow:#ff79c6;--keyword-declarations:#a6e22e;--operator-color:#f92672;--number-color:#fd971f;--string-color:#a6e22e;--comment-color:#75715e;--identifier-color:#f8f8f2;}:root .dracula{--bg-color:#282a36;--text-color:#f8f8f2;--keyword-controlFlow:#ff79c6;--keyword-declarations:#50fa7b;--operator-color:#bd93f9;--number-color:#ffb86c;--string-color:#50fa7b;--comment-color:#6272a4;--identifier-color:#f8f8f2;}:root .atom-one-dark{--bg-color:#282c34;--text-color:#c9d1d9;--keyword-controlFlow:#66d9ef;--keyword-declarations:#98c379;--operator-color:#c678dd;--number-color:#f92672;--string-color:#98c379;--comment-color:#5c6370;--identifier-color:#c9d1d9;}:root .solarized-dark{--bg-color:#002b36;--text-color:#839496;--keyword-controlFlow:#b58900;--keyword-declarations:#cb4b16;--operator-color:#dc322f;--number-color:#2aa198;--string-color:#268bd2;--comment-color:#586e75;--identifier-color:#839496;}:root .vs-code-dark{--bg-color:#1e1e1e;--text-color:#d4d4d4;--keyword-controlFlow:#007acc;--keyword-declarations:#0097a7;--operator-color:#c763d2;--number-color:#cc6633;--string-color:#9cdcfe;--comment-color:#6a737d;--identifier-color:#d4d4d4;}:root .github{--bg-color:#f6f8fa;--text-color:#24292e;--keyword-controlFlow:#0366d6;--keyword-declarations:#0098ca;--operator-color:#d73a49;--number-color:#0098ca;--string-color:#032f62;--comment-color:#959da5;--identifier-color:#24292e;}:root .ayu-mirage{--bg-color:#1f2330;--text-color:#cdd6f4;--keyword-controlFlow:#f44785;--keyword-declarations:#a6e22e;--operator-color:#61afef;--number-color:#f99157;--string-color:#a6e22e;--comment-color:#7f848e;--identifier-color:#cdd6f4;}:root .one-dark-pro{--bg-color:#282c34;--text-color:#c9d1d9;--keyword-controlFlow:#569cd6;--keyword-declarations:#98c379;--operator-color:#c678dd;--number-color:#f92672;--string-color:#98c379;--comment-color:#5c6370;--identifier-color:#c9d1d9;}:root .nord{--bg-color:#2e3440;--text-color:#d8dee9;--keyword-controlFlow:#81a1c1;--keyword-declarations:#b48ead;--operator-color:#d08770;--number-color:#a3be8c;--string-color:#ebcb8b;--comment-color:#616e88;--identifier-color:#d8dee9;}.syntax-highlighter{background-color:var(--bg-color);color:var(--text-color);padding:16px;border-radius:8px;overflow-x:auto;font-family:\"Fira Code\",monospace;white-space:pre-wrap;line-height:1.5;tab-size:4;}.syntax-highlighter *,.token{margin:0;padding:0;box-sizing:border-box;}.token.keyword-controlFlow,.token.keyword-declarations{margin-right:5px;}.token.keyword-controlFlow{color:var(--keyword-controlFlow);}.token.keyword-declarations{color:var(--keyword-declarations);}.token.operator{color:var(--operator-color);}.token.number{color:var(--number-color);}.token.string{color:var(--string-color);}.token.comment{color:var(--comment-color);font-style:italic;}.token.identifier{color:var(--identifier-color);}";

// Define the types for the JSON structure

// Load the JSON file as typed data
var syntaxDataTyped = syntaxData;

// Define the props for the component

var SyntaxHighlighter = function SyntaxHighlighter(_ref) {
  var code = _ref.code,
    language = _ref.language,
    theme = _ref.theme;
  var languageRules = syntaxDataTyped.languages[language];
  if (!languageRules) {
    return /*#__PURE__*/React.createElement("pre", null, code); // Fallback if the language is not found
  }

  // Tokenize the input code string
  var tokenize = function tokenize(code) {
    var tokenRegex = /(\".*?\"|'.*?'|`.*?`|\/\/.*?$|\/\*[\s\S]*?\*\/|[\w$]+|[+\-*/=%!&|^~<>?:;.,{}[\]()\s]|[^\s])/gm;
    return code.match(tokenRegex) || [];
  };

  // Determine the type of a token
  var getTokenType = function getTokenType(token) {
    var _languageRules$operat, _languageRules$string, _languageRules$commen, _languageRules$commen2;
    // Check if the token matches any keywords
    if (languageRules.keywords) {
      for (var _i = 0, _Object$entries = Object.entries(languageRules.keywords); _i < _Object$entries.length; _i++) {
        var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
          category = _Object$entries$_i[0],
          keywords = _Object$entries$_i[1];
        if (keywords.includes(token)) {
          return "keyword-".concat(category);
        }
      }
    }

    // Check if it's an operator
    if ((_languageRules$operat = languageRules.operators) !== null && _languageRules$operat !== undefined && _languageRules$operat.includes(token)) {
      return "operator";
    }

    // Check if it's a number
    if (languageRules.numbers && new RegExp(languageRules.numbers).test(token)) {
      return "number";
    }

    // Check if it's a string
    if ((_languageRules$string = languageRules.strings) !== null && _languageRules$string !== undefined && _languageRules$string.delimiters.some(function (d) {
      return token.startsWith(d);
    })) {
      return "string";
    }

    // Check if it's a comment
    if (token.startsWith(((_languageRules$commen = languageRules.comments) === null || _languageRules$commen === undefined ? undefined : _languageRules$commen.singleLine) || "") || (_languageRules$commen2 = languageRules.comments) !== null && _languageRules$commen2 !== undefined && _languageRules$commen2.multiLine && token.startsWith(languageRules.comments.multiLine[0]) && token.endsWith(languageRules.comments.multiLine[1])) {
      return "comment";
    }
    return "identifier"; // Default to identifier
  };
  var formatCode = function formatCode(code) {
    var indentLevel = 0;
    var lines = code.split("\n");
    var formattedLines = lines.map(function (line) {
      line = line.trim();
      if (line === "") {
        return "\n"; // Keep empty lines
      }
      var indentation = "  ".repeat(indentLevel);

      // Adjust indent level for braces
      if (line.startsWith("}")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }

      // Format the current line
      var formattedLine = "".concat(indentation).concat(line);

      // Adjust indent level for opening braces
      if (line.endsWith("{") && !line.startsWith("catch")) {
        indentLevel++;
      }

      // Special handling for `catch` to align it with `try`
      if (line.startsWith("catch")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }

      // Adjust for closing braces at the end of the line
      if (line.endsWith("}") && !line.startsWith("}")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }
      return formattedLine;
    });

    // Remove extra empty lines
    return formattedLines.join("\n").replace(/\n{2,}/g, "\n");
  };

  // Highlight the code
  var highlightCode = function highlightCode(code) {
    var formattedCode = formatCode(code);
    var tokens = tokenize(formattedCode);
    return tokens.map(function (token, index) {
      var type = getTokenType(token);
      return /*#__PURE__*/React.createElement("span", {
        key: index,
        className: "token ".concat(type)
      }, token === "\t" ? /*#__PURE__*/React.createElement("span", {
        style: {
          marginLeft: "1em"
        }
      }) : token);
    });
  };
  return /*#__PURE__*/React.createElement("pre", {
    className: "syntax-highlighter ".concat(language, " ").concat(theme)
  }, /*#__PURE__*/React.createElement("code", null, highlightCode(code)));
};

exports.SyntaxHighlighter = SyntaxHighlighter;
